#!/usr/bin/perl
# Inserts metadata from .meta files into the mysql database
# $Id: metadata_harvest.pl,v 1.5 2008/12/25 01:56:03 raeburn Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#


###############################################################################
##                                                                           ##
## ORGANIZATION OF THIS PERL CGI SCRIPT                                      ##
##                                                                           ##
## 1. Status of this code                                                    ##
## 2. Purpose and description of program                                     ##
## 3. Modules used by this script                                            ##
## 4. Print MIME Content-type and other initialization                       ##
## 5. Make sure database can be accessed and that this is a library server   ##
## 6. Loop through database records and print out keywords                   ##
##                                                                           ##
###############################################################################

# --------------------------------------------------------- Status of this code
#
# 1=horrible 2=poor 3=fair 4=good 5=excellent
# Organization 5
# Functionality 4
# Has it been tested? 4
#

# ------------------------------------------ Purpose and description of program
#
# This program outputs one line per database entry.
# The line is to be a list of keywords separated by commas.
# The file is to be output as a text file on a browser (text/plain).
# This provides initial data by which to study common and uncommon
# keywords being used.
# Note that the authoritative copy of metadata "keywords" is in the
# .meta files that are native to the library server.  We rely
# on the assumption that it is okay to use the MySQL server (which
# should reflect this information) instead.  This is a speedier approach.

# ------------------------------------------------- Modules used by this script

$|=1;
use strict;
use DBI;
use lib '/home/httpd/lib/perl/';
use Apache::lonlocal;
use LONCAPA::Configuration;
use LONCAPA::loncgi;
use LONCAPA::lonauthcgi;

# ---------------------------- Print MIME Content-type and other initialization
print 'Content-type: text/plain'."\n\n";

&main();

sub main {
    if (!&LONCAPA::lonauthcgi::check_ipbased_access('metadata_harvest')) {
        if (!&LONCAPA::loncgi::check_cookie_and_load_env()) {
            &Apache::lonlocal::get_language_handle();
            print(&LONCAPA::loncgi::missing_cookie_msg());
            return;
        }

        if (!&LONCAPA::lonauthcgi::can_view('metadata_harvest')) {
            &Apache::lonlocal::get_language_handle();
            print(&LONCAPA::lonauthcgi::unauthorized_msg('metadata_harvest'));
            return;
        }
    }

    &Apache::lonlocal::get_language_handle();

# --- Make sure that database can be accessed and that this is a library server
# library server test

# By default, loncapa_apache.conf is also read by the read_conf subroutine.
    my $perlvarref=LONCAPA::Configuration::read_conf('loncapa.conf');
    my %perlvar=%{$perlvarref};
    undef($perlvarref);

    unless ($perlvar{'lonRole'} eq 'library') {
        print(&Apache::lonlocal::mt('This can only be run on a library server!')."\n");
        return;
    }
# database test
    my $dbh;
    {
        unless (
	        $dbh = DBI->connect("DBI:mysql:loncapa","www",
				    $perlvar{'lonSqlAccess'},
				    { RaiseError =>0,PrintError=>0})
	        ) { 
	    print "Cannot connect to database!\n";
	    return;
        }
    }
    %perlvar=(); # undefine it

# ------------------------ Loop through database records and print out keywords
    my $sth=$dbh->prepare("select * from metadata");
    $sth->execute();
    my @row;
    while (@row=$sth->fetchrow_array) {
        for (my $i=0;$i<=$#row;$i++) {
            $row[$i]=~s/\n/ /g;
            $row[$i]=~s/\|/ /g;
        }
        print join('|',@row)."\n";
    }

# --------------------------------------------------- Close database connection
    $dbh->disconnect();
    return;
}
